@@ -13,9 +13,7 @@ We can also move the implementation to C++ if desired.
 """
 
 import datetime
-import inspect
 import mimetypes
-import platform
 import shutil
 from collections.abc import KeysView, MutableMapping
 from decimal import Decimal
@@ -24,7 +22,6 @@ from pathlib import Path
 from subprocess import run
 from tempfile import NamedTemporaryFile
 from typing import (
-    Any,
     BinaryIO,
     Callable,
     ItemsView,
@@ -32,7 +29,6 @@ from typing import (
     List,
     Optional,
     Tuple,
-    Type,
     TypeVar,
     Union,
     ValuesView,
@@ -40,6 +36,7 @@ from typing import (
 from warnings import warn
 
 from . import Array, Dictionary, Name, Object, Page, Pdf, Stream
+from ._augments import augment_override_cpp, augments
 from ._qpdf import (
     AccessMode,
     AttachedFile,
@@ -64,127 +61,6 @@ __all__ = []
 Numeric = TypeVar('Numeric', int, float, Decimal)
 
 
-def augment_override_cpp(fn):
-    """This Python function should replace the C++ implementation, if there is one."""
-    fn._augment_override_cpp = True
-    return fn
-
-
-def augment_if_no_cpp(fn):
-    """This Python function provides a Python implementation if no C++ implementation exists."""
-    fn._augment_if_no_cpp = True
-    return fn
-
-
-def _is_inherited_method(meth):
-    # Augmenting a C++ with a method that cls inherits from the Python
-    # object is never what we want.
-    return meth.__qualname__.startswith('object.')
-
-
-def _is_augmentable(m):
-    return (
-        inspect.isfunction(m) and not _is_inherited_method(m)
-    ) or inspect.isdatadescriptor(m)
-
-
-def augments(cls_cpp: Type[Any]):
-    """Attach methods of a Python support class to an existing class
-
-    This monkeypatches all methods defined in the support class onto an
-    existing class. Example:
-
-    .. code-block:: python
-
-        @augments(ClassDefinedInCpp)
-        class SupportClass:
-            def foo(self):
-                pass
-
-    The Python method 'foo' will be monkeypatched on ClassDefinedInCpp. SupportClass
-    has no meaning on its own and should not be used, but gets returned from
-    this function so IDE code inspection doesn't get too confused.
-
-    We don't subclass because it's much more convenient to monkeypatch Python
-    methods onto the existing Python binding of the C++ class. For one thing,
-    this allows the implementation to be moved from Python to C++ or vice
-    versa. It saves having to implement an intermediate Python subclass and then
-    ensures that the C++ superclass never 'leaks' to pikepdf users. Finally,
-    wrapper classes and subclasses can become problematic if the call stack
-    crosses the C++/Python boundary multiple times.
-
-    Any existing methods may be used, regardless of whether they are defined
-    elsewhere in the support class or in the target class.
-
-    For data fields to work, the target class must be
-    tagged ``py::dynamic_attr`` in pybind11.
-
-    Strictly, the target class does not have to be C++ or derived from pybind11.
-    This works on pure Python classes too.
-
-    THIS DOES NOT work for class methods.
-
-    (Alternative ideas: https://github.com/pybind/pybind11/issues/1074)
-    """
-
-    OVERRIDE_WHITELIST = {'__eq__', '__hash__', '__repr__'}
-    if platform.python_implementation() == 'PyPy':
-        # Either PyPy or pybind11's interface to PyPy automatically adds a __getattr__
-        OVERRIDE_WHITELIST |= {'__getattr__'}  # pragma: no cover
-
-    def class_augment(cls, cls_cpp=cls_cpp):
-
-        # inspect.getmembers has different behavior on PyPy - in particular it seems
-        # that a typical PyPy class like cls will have more methods that it considers
-        # methods than CPython does. Our predicate should take care of this.
-        for name, member in inspect.getmembers(cls, predicate=_is_augmentable):
-            if name == '__weakref__':
-                continue
-            if (
-                hasattr(cls_cpp, name)
-                and hasattr(cls, name)
-                and name not in getattr(cls, '__abstractmethods__', set())
-                and name not in OVERRIDE_WHITELIST
-                and not getattr(getattr(cls, name), '_augment_override_cpp', False)
-            ):
-                if getattr(getattr(cls, name), '_augment_if_no_cpp', False):
-                    # If tagged as "augment if no C++", we are supported older pybind11
-                    # versions that may not define some functions. If pybind11 provides
-                    # the required function, don't override it.
-                    continue
-
-                # If the original C++ class and Python support class both define the
-                # same name, we generally have a conflict, because this is augmentation
-                # not inheritance. However, if the method provided by the support class
-                # is an abstract method, then we can consider the C++ version the
-                # implementation. Also, pybind11 provides defaults for __eq__,
-                # __hash__ and __repr__ that we often do want to override directly.
-
-                raise RuntimeError(  # pragma: no cover
-                    f"C++ {cls_cpp} and Python {cls} both define the same "
-                    f"non-abstract method {name}: "
-                    f"{getattr(cls_cpp, name, '')!r}, "
-                    f"{getattr(cls, name, '')!r}"
-                )
-            if inspect.isfunction(member):
-                setattr(cls_cpp, name, member)
-                installed_member = getattr(cls_cpp, name)
-                installed_member.__qualname__ = member.__qualname__.replace(
-                    cls.__name__, cls_cpp.__name__
-                )
-            elif inspect.isdatadescriptor(member):
-                setattr(cls_cpp, name, member)
-
-        def disable_init(self):
-            # Prevent initialization of the support class
-            raise NotImplementedError(self.__class__.__name__ + '.__init__')
-
-        cls.__init__ = disable_init
-        return cls
-
-    return class_augment
-
-
 def _single_page_pdf(page) -> bytes:
     """Construct a single page PDF from the provided page in memory."""
     pdf = Pdf.new()
@@ -1002,7 +878,11 @@ class Extend_Page:
 
     @property
     def images(self) -> _ObjectMapping:
-        """Return all images associated with this page."""
+        """Return all regular images associated with this page.
+
+        This method does not recurse into Form XObjects and does not
+        attempt to find inline images.
+        """
         return self._images
 
     @property
