@@ -60,21 +60,24 @@ void init_page(py::module_ &m)
         }))
         .def(
             "__copy__", [](QPDFPageObjectHelper &poh) { return poh.shallowCopyPage(); })
-        .def_property_readonly("_images", &QPDFPageObjectHelper::getPageImages)
+        .def_property_readonly("_images", &QPDFPageObjectHelper::getImages)
         .def("_get_mediabox", &QPDFPageObjectHelper::getMediaBox)
         .def("_get_cropbox", &QPDFPageObjectHelper::getCropBox)
         .def("_get_trimbox", &QPDFPageObjectHelper::getTrimBox)
         .def(
             "externalize_inline_images",
-            [](QPDFPageObjectHelper &poh, size_t min_size = 0) {
-                return poh.externalizeInlineImages(min_size);
+            [](QPDFPageObjectHelper &poh, size_t min_size = 0, bool shallow = false) {
+                return poh.externalizeInlineImages(min_size, shallow);
             },
             py::arg("min_size") = 0,
+            py::arg("shallow")  = false,
             R"~~~(
                 Convert inlines image to normal (external) images.
 
                 Args:
                     min_size (int): minimum size in bytes
+                    shallow (bool): If False, recurse into nested Form XObjects.
+                        If True, do not recurse.
             )~~~")
         .def("rotate",
             &QPDFPageObjectHelper::rotatePage,
@@ -209,7 +212,7 @@ void init_page(py::module_ &m)
             [](QPDFPageObjectHelper &poh,
                 QPDFObjectHandle::TokenFilter &tf) -> py::bytes {
                 Pl_Buffer pl_buffer("filter_page");
-                poh.filterPageContents(&tf, &pl_buffer);
+                poh.filterContents(&tf, &pl_buffer);
 
                 PointerHolder<Buffer> buf(pl_buffer.getBuffer());
                 auto data = reinterpret_cast<const char *>(buf->getBuffer());
@@ -265,7 +268,7 @@ void init_page(py::module_ &m)
         .def(
             "parse_contents",
             [](QPDFPageObjectHelper &poh, PyParserCallbacks &parsercallbacks) {
-                poh.parsePageContents(&parsercallbacks);
+                poh.parseContents(&parsercallbacks);
             },
             R"~~~(
                 Parse a page's content streams using a :class:`pikepdf.StreamParser`.
