@@ -11,16 +11,6 @@
 
 #include "utils.h"
 
-/* POSIX functions on Windows have a leading underscore
- */
-#if defined(_WIN32)
-#   define posix_fdopen _fdopen
-#   define posix_close  _close
-#else
-#   define posix_fdopen fdopen
-#   define posix_close  close
-#endif
-
 /* Convert a Python object to a filesystem encoded path
  * Use Python's os.fspath() which accepts os.PathLike (str, bytes, pathlib.Path)
  * and returns bytes encoded in the filesystem encoding.
