@@ -6,6 +6,8 @@ import pytest
 
 import pikepdf
 from pikepdf import (
+    ContentStreamInlineImage,
+    ContentStreamInstruction,
     Dictionary,
     Name,
     Object,
@@ -22,14 +24,24 @@ from pikepdf import (
 from pikepdf._qpdf import StreamParser
 from pikepdf.models import PdfParsingError
 
-# pylint: disable=useless-super-delegation
+# pylint: disable=useless-super-delegation,redefined-outer-name
+
+
+@pytest.fixture
+def graph(resources):
+    yield Pdf.open(resources / 'graph.pdf')
+
+
+@pytest.fixture
+def inline(resources):
+    yield Pdf.open(resources / 'image-mono-inline.pdf')
 
 
 class PrintParser(StreamParser):
     def __init__(self):
         super().__init__()
 
-    def handle_object(self, obj):
+    def handle_object(self, obj, *_args):
         print(repr(obj))
 
     def handle_eof(self):
@@ -40,7 +52,7 @@ class ExceptionParser(StreamParser):
     def __init__(self):
         super().__init__()
 
-    def handle_object(self, obj):  # pylint: disable=unused-argument
+    def handle_object(self, obj, *_args):  # pylint: disable=unused-argument
         raise ValueError("I take exception to this")
 
     def handle_eof(self):
@@ -83,15 +95,13 @@ def slow_unparse_content_stream(instructions):
     return b'\n'.join(for_each_instruction())
 
 
-def test_open_pdf(resources):
-    pdf = Pdf.open(resources / 'graph.pdf')
-    page = pdf.pages[0]
-    Object._parse_stream(page, PrintParser())
+def test_open_pdf(graph):
+    page = graph.pages[0]
+    Object._parse_stream(page.obj, PrintParser())
 
 
-def test_parser_exception(resources):
-    pdf = Pdf.open(resources / 'graph.pdf')
-    stream = pdf.pages[0]['/Contents']
+def test_parser_exception(graph):
+    stream = graph.pages[0]['/Contents']
     with pytest.raises(ValueError):
         Object._parse_stream(stream, ExceptionParser())
 
@@ -106,25 +116,24 @@ def test_text_filter(resources, outdir):
     )
     assert proc.stdout.strip() != '', "Need input test file that contains text"
 
-    pdf = Pdf.open(input_pdf)
-    page = pdf.pages[0]
+    with Pdf.open(input_pdf) as pdf:
+        page = pdf.pages[0]
 
-    keep = []
-    for operands, command in parse_content_stream(
-        page, """TJ Tj ' " BT ET Td TD Tm T* Tc Tw Tz TL Tf Tr Ts"""
-    ):
-        if command == Operator('Tj'):
-            print("skipping Tj")
-            continue
-        keep.append((operands, command))
+        keep = []
+        for operands, command in parse_content_stream(
+            page, """TJ Tj ' " BT ET Td TD Tm T* Tc Tw Tz TL Tf Tr Ts"""
+        ):
+            if command == Operator('Tj'):
+                print("skipping Tj")
+                continue
+            keep.append((operands, command))
 
-    new_stream = Stream(pdf, pikepdf.unparse_content_stream(keep))
-    print(new_stream.read_bytes())  # pylint: disable=no-member
-    page['/Contents'] = new_stream
-    page['/Rotate'] = 90
+        new_stream = Stream(pdf, pikepdf.unparse_content_stream(keep))
+        print(new_stream.read_bytes())  # pylint: disable=no-member
+        page['/Contents'] = new_stream
+        page['/Rotate'] = 90
 
-    pdf.save(outdir / 'notext.pdf', True)
-    pdf.close()
+        pdf.save(outdir / 'notext.pdf', static_id=True)
 
     proc = run(
         ['pdftotext', str(outdir / 'notext.pdf'), '-'],
@@ -161,10 +170,10 @@ def test_invalid_stream_object():
 #     ],
 # )
 # def test_has_text(resources, test_file, expected):
-#     pdf = Pdf.open(resources / test_file)
-#     for p in pdf.pages:
-#         page = Page(p)
-#         assert page.has_text() == expected
+#     with Pdf.open(resources / test_file) as pdf:
+#         for p in pdf.pages:
+#             page = p
+#             assert page.has_text() == expected
 
 
 def test_unparse_cs():
@@ -183,10 +192,35 @@ def test_unparse_failure():
 
 
 def test_parse_xobject(resources):
-    pdf = Pdf.open(resources / 'formxobject.pdf')
-    form1 = pdf.pages[0].Resources.XObject.Form1
-    instructions = parse_content_stream(form1)
-    assert instructions[0][1] == Operator('cm')
+    with Pdf.open(resources / 'formxobject.pdf') as pdf:
+        form1 = pdf.pages[0].Resources.XObject.Form1
+        instructions = parse_content_stream(form1)
+        assert instructions[0][1] == Operator('cm')
+
+
+def test_parse_results(inline):
+    p0 = inline.pages[0]
+    cmds = parse_content_stream(p0)
+    assert isinstance(cmds[0], ContentStreamInstruction)
+    csi = cmds[0]
+    assert isinstance(csi.operands, _qpdf._ObjectList)
+    assert isinstance(csi.operator, Operator)
+    assert 'Operator' in repr(csi)
+
+    assert ContentStreamInstruction(cmds[0]).operator == cmds[0].operator
+
+    for cmd in cmds:
+        if isinstance(cmd, ContentStreamInlineImage):
+            assert cmd.operator == Operator("INLINE IMAGE")
+            assert isinstance(cmd.operands[0], PdfInlineImage)
+            assert 'INLINE' in repr(cmd)
+            assert cmd.operands[0] == cmd.iimage
+
+
+def test_build_instructions():
+    cs = ContentStreamInstruction([1, 0, 0, 1, 0, 0], Operator('cm'))
+    assert 'cm' in repr(cs)
+    assert unparse_content_stream([cs]) == b'1 0 0 1 0 0 cm'
 
 
 def test_unparse_interpret_operator():
@@ -203,13 +237,12 @@ def test_unparse_interpret_operator():
     )
 
 
-def test_unparse_inline(resources):
-    with Pdf.open(resources / 'image-mono-inline.pdf') as pdf:
-        p0 = pdf.pages[0]
-        cmds = parse_content_stream(p0)
-        unparsed = unparse_content_stream(cmds)
-        assert b'BI' in unparsed
-        assert unparsed == slow_unparse_content_stream(cmds)
+def test_unparse_inline(inline):
+    p0 = inline.pages[0]
+    cmds = parse_content_stream(p0)
+    unparsed = unparse_content_stream(cmds)
+    assert b'BI' in unparsed
+    assert unparsed == slow_unparse_content_stream(cmds)
 
 
 def test_unparse_invalid_inline_image():
@@ -219,15 +252,57 @@ def test_unparse_invalid_inline_image():
         unparse_content_stream(instructions)
 
 
+def test_inline_copy(inline):
+    for instr in parse_content_stream(inline.pages[0].Contents):
+        if not isinstance(instr, ContentStreamInlineImage):
+            continue
+        csiimage = instr
+        _copy_of_csiimage = ContentStreamInlineImage(csiimage)
+        new_iimage = ContentStreamInlineImage(csiimage.iimage)
+        assert unparse_content_stream([new_iimage]).startswith(b'BI')
+
+
+def test_end_inline_parse():
+    pdf = pikepdf.new()
+    pdf.add_blank_page(page_size=(1000, 1000))
+    stream = b"""
+        q 200 0 0 200 500 500 cm
+        BI
+        /W 1
+        /H 1 
+        /BPC 8
+        /CS /RGB
+        ID \x80\x80\x80
+        EI Q
+        q 300 0 0 300 500 200 cm
+        BI
+        /W 2
+        /H 2
+        /BPC 8
+        /CS /RGB
+        ID \xff\x00\x00\x00\xff\x00\x00\xff\x00\x00\x00\xff
+        EI Q
+        """
+    pdf.pages[0].Contents = pdf.make_stream(stream)
+    cs = parse_content_stream(pdf.pages[0])
+    assert unparse_content_stream(cs).split() == stream.split()
+
+
 class TestMalformedContentStreamInstructions:
     def test_rejects_not_list_of_pairs(self):
         with pytest.raises(PdfParsingError):
             unparse_content_stream([(1, 2, 3)])
 
-    def test_rejects_not_operator(self):
-        with pytest.raises(PdfParsingError):
+    def test_rejects_not_castable_to_object(self):
+        with pytest.raises(PdfParsingError, match="While unparsing"):
             unparse_content_stream([(['one', 'two'], 42)])  # 42 is not an operator
 
+    def test_rejects_not_operator(self):
+        with pytest.raises(PdfParsingError, match="While unparsing"):
+            unparse_content_stream(
+                [(['one', 'two'], Name.FortyTwo)]
+            )  # Name is not an operator
+
     def test_rejects_inline_image_missing(self):
         with pytest.raises(PdfParsingError):
             unparse_content_stream(
